#!/usr/bin/perl -w
#---------------------------------------------------------------------
#/* Copyright (C) 2000-2006 MySQL AB
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# Version 1.1
#------------------------------------------------------------------------
# Purpose: This application creates and loads the database, tables and 
#          Stored procdures for the TPC-B. The Storage engine currently
#          can be NDBCluster, InnoDB, or MyISAM, but is easly exanded
#          to use other storage engines.
#------------------------------------------------------------------------
#
#-----------------------------------------------
#             Perl Includes 
#-----------------------------------------------
use DBI;
#-----------------------------------------------
#   Globals Vars  
#-----------------------------------------------
our $m_host=''; 
our $m_port='';
our $m_user='';
our $m_pass='';
our $dbhM='';
our $aid = 0;
our $bid = 0;
our $tid = 0;
our $balance = 0;
our $accounts = 100000; #/should be changed for bigger databases
our $branches = 1000;   #/should be changed for bigger databases
our $c_only = 0;
our $engine_type = 'NDB';
our $DiskData = 0;
#-------------------------------------------------
#   Sub Pototypes 
#-------------------------------------------------
sub main ();
sub CollectCommandPromptInfo ();
sub ConnectToDatabases ();
sub DisconnectFromDatabase ();
sub CreateDB ();
sub Create_Tables ();
sub Create_Procedures ();
sub Load_Tables ();
#--------------------------------------------------
#  Program Main 
#--------------------------------------------------
main();

sub main ()
{
  CollectCommandPromptInfo();
  ConnectToDatabases();
  CreateDB();
  Create_Tables();
  Create_Procedures();
  if ($c_only == 0) 
  {
    Load_Tables;
  }
  DisconnectFromDatabase;
}
#--------------------------------
#  Collect Command Prompt Info 
#--------------------------------
sub CollectCommandPromptInfo ()
{
  ### Check that user has supplied correct number of command line args
die "Usage:\n
     Load_tpcb.pl <host> <port> <user> <pass> <engine>\n 
                   --create-only\n\n
    Note: engine == ndb, ndb_dd(Disk Data), ndbcluster, InnoDB, MyISAM\n
    --create-only creates the database and tables but load no data.
    Note: First 5 arguments must be passed. Use BLANK for NULL passwords\n" 
  unless @ARGV >= 5;

  $m_host = $ARGV[0]; 
  $m_port = $ARGV[1];
  $m_user = $ARGV[2];
  $m_pass = $ARGV[3];
  $engine_type = $ARGV[4];
  if(@ARGV >= 6){
    if ($ARGV[5] eq '--create-only'){
      $c_only = 1;
    }
    else{
      die "Usage:\n
           Load_tpcb.pl <host> <port> <user> <pass> <engine>\n 
                  --create-only\n\n
           Note: engine == ndb, ndbcluster, InnoDB, MyISAM\n
           --create-only (optional) creates the database and tables only.
           Note: First 5 arguments must be passed. Use BLANK for NULL passwords\n" 
    }
  } 

  if ($engine_type ne 'ndb' && $engine_type ne 'ndbcluster'
      && $engine_type ne 'InnoDB' && $engine_type ne 'MyISAM' 
      && $engine_type ne 'ndb_dd'){
    die "Usage:\n
         Load_tpcb.pl <host> <port> <user> <pass> <engine>\n 
                      --create-only\n\n
         Note: engine == ndb, ndb_dd(disk data),ndbcluster, InnoDB, MyISAM\n
         --create-only (optional) creates the database and tables only.
         Note: First 5 arguments must be passed. Use BLANK for NULL passwords\n" 
  } 
  if ($m_pass eq "BLANK") { $m_pass = '';}
  if ($engine_type eq 'ndb_dd'){$engine_type = "ndb"; $DiskData = 1;}
}

#---------------------------------------------
#  Connect to database 
#---------------------------------------------
sub ConnectToDatabases ()
{
  $dbhM = DBI->connect("dbi:mysql:database=mysql;host=$m_host;port=$m_port", 
                       "$m_user", "$m_pass")
    or die "Can't connect to Master Cluster MySQL process! Error: $DBI::errstr\n";
 
  print " Connected to database.\n";
}

#-----------------------------------------------
#  Disconnect from database 
#-----------------------------------------------
sub DisconnectFromDatabase ()
{
  $dbhM->disconnect
    or warn " Disconnection failed: $DBI::errstr\n";
  print "Disconnected from database.\n";
}

#-------------------------------------------------
#  Create database 
#-------------------------------------------------
sub CreateDB ()
{

  $sth = $dbhM->prepare("DROP DATABASE IF EXISTS TPCB;")
	or die "Prepare error: ", $dbhM->errstr;
  $sth->execute();#/Don't care if returns success. No "die" needed
  $sth->finish();

  $sth = $dbhM->prepare("CREATE DATABASE TPCB;")
    or die "Prepare error: ", $dbhM->errstr;
  $sth->execute()
    or die "Create DB Error: ", $sth->errstr;
  $sth->finish();

  $sth = $dbhM->prepare("USE TPCB;")
    or die "Prepare error: ", $dbhM->errstr;
  $sth->execute()
    or die "USE DB Error: ", $sth->errstr;
  $sth->finish();

  if($DiskData == 1){
    $sth = $dbhM->prepare("CREATE LOGFILE GROUP TPCB_LOG
                           ADD UNDOFILE './tpcb_log/undofile.dat'
                           INITIAL_SIZE 150M
                           UNDO_BUFFER_SIZE = 1M
                           ENGINE=NDB;")
      or die "Prepare CREATE LOGFILE GROUP error: ", $dbhM->errstr;
    $sth->execute()
      or die "CREATE LOGFILE GROUP error: ", $sth->errstr;
    $sth->finish();
  
    $sth = $dbhM->prepare("ALTER LOGFILE GROUP TPCB_LOG
                           ADD UNDOFILE './tpcb_log/undofile2.dat'
                           INITIAL_SIZE 150M
                           ENGINE=NDB;")
      or die "Prepare ALTER LOGFILE GROUP error: ", $dbhM->errstr;
    $sth->execute()
      or die "ALTER LOGFILE GROUP error: ", $sth->errstr;
    $sth->finish();
 
    $sth = $dbhM->prepare("CREATE TABLESPACE TPCB_TS
                           ADD DATAFILE './tpcb_ts/datafile.dat'
                           USE LOGFILE GROUP TPCB_LOG
                           INITIAL_SIZE 5M
                           ENGINE=NDB;")
      or die "Prepare CREATE TABLESPACE error: ", $dbhM->errstr;
    $sth->execute()
      or die "CREATE TABLESPACE error: ", $sth->errstr;
    $sth->finish();
  }

  print "Database Created.\n";
}

#------------------------------------------------------
#  Create tables 
#------------------------------------------------------
sub Create_Tables ()
{

  $sth = $dbhM->prepare("DROP TABLE IF EXISTS account;")
	or die "Prepare drop account error: ", $dbhM->errstr;
  $sth->execute();#/Don't care if returns success. No die needed
  $sth->finish();

  $sth = $dbhM->prepare("DROP TABLE IF EXISTS branch;")
	or die "Prepare drop branch error: ", $dbhM->errstr;
  $sth->execute();#/Don't care if returns success. No die needed
  $sth->finish();

  $sth = $dbhM->prepare("DROP TABLE IF EXISTS teller;")
	or die "Prepare drop teller error: ", $dbhM->errstr;
  $sth->execute();#/Don't care if returns success. No die needed
  $sth->finish();

  $sth = $dbhM->prepare("DROP TABLE IF EXISTS history;")
	or die "Prepare drop history error: ", $dbhM->errstr;
  $sth->execute();#/Don't care if returns success. No die needed
  $sth->finish();

  $sth = $dbhM->prepare("DROP TABLE IF EXISTS trans;")
        or die "Prepare drop trans error: ", $dbhM->errstr;
  $sth->execute();#/Don't care if returns success. No die needed
  $sth->finish();

  print "Creating Account Table\n";

  if($DiskData == 0){
    $sth = $dbhM->prepare("CREATE TABLE account (aid INT, bid INT, balance DECIMAL(8,2), filler CHAR(80), adesc TEXT, PRIMARY KEY (aid))ENGINE=$engine_type;")
      or die "Prepare create account error: ", $dbhM->errstr;
    $sth->execute()
      or die "Create account table Error: ", $sth->errstr;
    $sth->finish();
  }
  else{
    $sth = $dbhM->prepare("CREATE TABLE account (aid INT, bid INT, balance DECIMAL(8,2), filler CHAR(80), PRIMARY KEY (aid)) TABLESPACE TPCB_TS STORAGE DISK ENGINE=$engine_type;")
      or die "Prepare create account error: ", $dbhM->errstr;
    $sth->execute()
      or die "Create account table Error: ", $sth->errstr;
    $sth->finish();
  }

  print "Creating Branch Table\n";
  if($DiskData == 0){
    $sth = $dbhM->prepare("CREATE TABLE branch (bid INT, balance DECIMAL(8,2), filler CHAR(80), PRIMARY KEY (bid))ENGINE=$engine_type;")
      or die "Prepare create branch error: ", $dbhM->errstr;
    $sth->execute()
      or die "Create branch table Error: ", $sth->errstr;
    $sth->finish();
  }
  else{
    $sth = $dbhM->prepare("CREATE TABLE branch (bid INT, balance DECIMAL(8,2), filler CHAR(80), PRIMARY KEY (bid)) TABLESPACE TPCB_TS STORAGE DISK ENGINE=$engine_type;")
      or die "Prepare create branch error: ", $dbhM->errstr;
    $sth->execute()
      or die "Create branch table Error: ", $sth->errstr;
    $sth->finish();
  }

  print "Creating Teller Table\n";
  if($DiskData == 0){
     $sth = $dbhM->prepare("CREATE TABLE teller (tid INT, bid INT,  balance DECIMAL(8,2), filler CHAR(80), PRIMARY KEY (tid))ENGINE=$engine_type;")
       or die "Prepare create teller error: ", $dbhM->errstr;
     $sth->execute()
       or die "Create teller table Error: ", $sth->errstr;
     $sth->finish();
  }
  else{
     $sth = $dbhM->prepare("CREATE TABLE teller (tid INT, bid INT,  balance DECIMAL(8,2), filler CHAR(80), PRIMARY KEY (tid)) TABLESPACE TPCB_TS STORAGE DISK ENGINE=$engine_type;")
       or die "Prepare create teller error: ", $dbhM->errstr;
     $sth->execute()
       or die "Create teller table Error: ", $sth->errstr;
     $sth->finish();
  }

  print "Creating History Table\n";
  if($DiskData == 0){
    $sth = $dbhM->prepare("CREATE TABLE history (id MEDIUMINT NOT NULL AUTO_INCREMENT, aid INT, tid INT, bid INT, filler CHAR(80), PRIMARY KEY (id))ENGINE=$engine_type;")
      or die "Prepare create history error: ", $dbhM->errstr;
    $sth->execute()
      or die "Create Table Error: ", $sth->errstr;
    $sth->finish();
  }
  else{
     $sth = $dbhM->prepare("CREATE TABLE history (id MEDIUMINT NOT NULL AUTO_INCREMENT, aid INT, tid INT, bid INT, filler CHAR(80), PRIMARY KEY (id)) TABLESPACE TPCB_TS STORAGE DISK ENGINE=$engine_type;")       
       or die "Prepare create history error: ", $dbhM->errstr;
    $sth->execute()
      or die "Create Table Error: ", $sth->errstr;
    $sth->finish();
  }
  
  print "Creating Trans Table\n";
  $sth = $dbhM->prepare("CREATE TABLE trans (scriptId CHAR(10) NOT NULL, count INT, tps FLOAT, lastTransTime DATETIME, startTime DATETIME,PRIMARY KEY (scriptId))ENGINE=$engine_type;")
	or die "Prepare error: ", $dbhM->errstr;
  $sth->execute
 	or die "Create Table Error: ", $sth->errstr;
  $sth->finish;
  
  print "Tables Created.\n";
}

#--------------------------------------------------------
#  Create Stored Procedures 
#---------------------------------------------------------
sub Create_Procedures ()
{
  $sth = $dbhM->prepare("DROP PROCEDURE IF EXISTS TPCB.ExTrans;") 
         or die "Prepare drop ExTrans error: ", $dbhM->errstr;
  $sth->execute
        or die "Drop SP  Error: ", $sth->errstr;
  $sth->finish;

  $sth = $dbhM->prepare("
    CREATE PROCEDURE TPCB.ExTrans(IN acct INT, IN brch INT, IN tell INT, IN amt DECIMAL(10,2))
    BEGIN
     DECLARE bal DECIMAL(10,2) DEFAULT 0.0;
     DECLARE bbal DECIMAL(10,2) DEFAULT 0.0;
     DECLARE tbal DECIMAL(10,2) DEFAULT 0.0;

     START TRANSACTION;
     SELECT account.balance INTO bal FROM TPCB.account WHERE aid = acct;
     SELECT teller.balance INTO tbal FROM TPCB.teller WHERE tid = tell;
     SELECT branch.balance INTO bbal FROM TPCB.branch WHERE bid = brch;

     SET bal = bal + amt;
     SET bbal = bbal + amt;
     SET tbal = tbal + amt;

     UPDATE TPCB.account SET balance = bal, filler = 'account updated' WHERE aid = acct;
     UPDATE TPCB.branch SET balance = bbal, filler = 'branch updated'  WHERE bid = brch;
     UPDATE TPCB.teller SET balance = tbal, filler = 'teller updated'  WHERE tid = tell;
     COMMIT;
    END;")or die "Prepare ExTrans error: ", $dbhM->errstr;

  $sth->execute
        or die "Create ExTrans Procedure Error: ", $sth->errstr;
  $sth->finish;

  $sth = $dbhM->prepare("DROP PROCEDURE IF EXISTS TPCB.HistInsert;") 
         or die "Prepare Drop HistInsert error: ", $dbhM->errstr;
  $sth->execute
        or die "Drop HI SP  Error: ", $sth->errstr;
  $sth->finish;

 $sth = $dbhM->prepare("CREATE PROCEDURE TPCB.HistInsert(IN acct INT, IN brch INT, IN tell INT)
    BEGIN
     START TRANSACTION;
     INSERT into TPCB.history (aid,tid,bid, filler) VALUES(acct,tell,brch, 'MoreMoney');
     COMMIT;
    END;") or die "Prepare History Insert SP error: ", $dbhM->errstr;

  $sth->execute
        or die "Create HI Procedure Error: ", $sth->errstr;
  $sth->finish;
  
 $sth = $dbhM->prepare("DROP PROCEDURE IF EXISTS TPCB.UpdateTrans;") 
         or die "Prepare Drop Update Trans  error: ", $dbhM->errstr;
  $sth->execute
        or die "Drop Update Trans SP  Error: ", $sth->errstr;
  $sth->finish;

 $sth = $dbhM->prepare("CREATE PROCEDURE TPCB.UpdateTrans(IN counter INT, IN tps FLOAT, IN id CHAR(10))
    BEGIN
     START TRANSACTION;
     UPDATE TPCB.trans set count = counter, lastTransTime = now(), TPS = tps where scriptId = id;
     COMMIT;
    END;") or die "Prepare Update Trans SP error: ", $dbhM->errstr;

  $sth->execute
        or die "Create Update Trans Procedure Error: ", $sth->errstr;
  $sth->finish;
}
#-----------------------------------------------
#  Load tables 
#-----------------------------------------------
sub Load_Tables ()
{
  print "Preparing to load tables\n";
  print "Loading accounts table -- Please wait\n";
  $sth = $dbhM->prepare("INSERT into account VALUES(?, ?, ?, ?, ?);")
	or die "Prepare insert into account error: ", $dbhM->errstr;
  
  while($bid < $branches){
    $bid++;
    $i = 0;
    while($i < ($accounts/$branches)){
      $i++;
      $aid++;
#      print "aid = $aid\n";
#      print "bid = $bid\n";
      $sth->bind_param(1, $aid);
      $sth->bind_param(2, $bid);
      $sth->bind_param(3, $balance);
      $sth->bind_param(4, "Going fishing");
      $sth->bind_param(5, "REPEAT('A',2000)");
      $sth->execute
        or die "insert into account Error: ", $sth->errstr;
     }
  }
  $sth->finish;

  $sth = $dbhM->prepare("INSERT into branch values(?, ?, ?);")
	or die "Prepare insert into account error: ", $dbhM->errstr;
  $bid = 0;
  print "\nLoading Branch Table ";
  while($bid < $branches){
    $bid++;
    $sth->bind_param(1, $bid);
    $sth->bind_param(2, $balance);
    $sth->bind_param(3, "Going on Vacation");
    $sth->execute
      or die "insert into branch Error: ", $sth->errstr;
  }	
  $sth->finish;

  $sth = $dbhM->prepare("INSERT into teller values(?, ?, ?, ?);")
	or die "Prepare insert into account error: ", $dbhM->errstr;
  $bid=0;
  print "\nLoading Teller Table ";
  while($bid < $branches){
    $bid++;
    $i=0;
    while($i < 20){
      $i++;
      $tid++;
      $sth->bind_param(1, $tid);
      $sth->bind_param(2, $bid);
      $sth->bind_param(3, $balance);
      $sth->bind_param(4, "I have to work");
      $sth->execute
 	or die "insert into teller Error: ", $sth->errstr;
    }
  }
  $sth->finish;
  print "Tables have been loaded!\n";
}
###################  End of script #############
